@@ -39,7 +39,8 @@
 using namespace magics;
 
 
-ViewNode::ViewNode() : viewTransformation_(0), animation_("basic"), vaxis_(2.), haxis_(1.),  rules_(0), legend_(0), needLegend_(false)
+ViewNode::ViewNode() : viewTransformation_(0), animation_("basic"), vaxis_(2.), haxis_(1.),  rules_(0), legend_(0), 
+			needLegend_(false), drawing_background_colour_("white")
 {
 	static int i = 0;
 	ostringstream n;
@@ -48,6 +49,7 @@ ViewNode::ViewNode() : viewTransformation_(0), animation_("basic"), vaxis_(2.),
 	i++;
 	layout_ = new Layout();
 	layout_->name(name_);
+
 } 
 
 
@@ -170,13 +172,13 @@ void ViewNode::prepareLayout(SceneLayer& tree)
 	frameHelper_->heightResolution(heightResolution()*width/100);
 
 	frameHelper_->frame(*layout_);
-	frameHelper_->blankIt();
-	frameHelper_->frameIt();
+	frameHelper_->backgroundColour(drawing_background_colour_);
+	
 	frameHelper_->clippIt(false);
 
-
+	//components_.push_back(frameHelper_); // first to draw the background if needed!
 	components_.push_back(drawing_);
-	components_.push_back(frameHelper_);
+	
 	helper.add(drawing_);
 
 	// Then the axis!
@@ -217,7 +219,7 @@ void ViewNode::prepareLayout(SceneLayer& tree)
 	helper.add(rightAxis_);
 		
 	
-	tree.legend(legend_);
+	legend_ = tree.legend(legend_);
 	if ( needLegend_ && legend_ )
 	{
 		if ( !legend_->positional() ) {
@@ -287,23 +289,29 @@ void ViewNode::visit(SceneLayer& tree)
 	// First the info about the animation overlay! 
 	// Basic instantiation of the Animation rules...
 	
+	
+	
+
 	if (!rules_) {
 		rules_ = MagTranslator<string, AnimationRules>()(lowerCase(animation_));
 		BasicSceneObject::visit(*rules_);
 	}
 
 	tree.rules(rules_);
-	// Here we checkeing for the legend!
+	// Here we checking for the legend!
 	needLegend_ = false;
 	for ( vector<BasicSceneObject*>::iterator item = items_.begin(); item != items_.end(); ++item)  {
 			needLegend_ = (*item)->needLegend();
 			if ( needLegend_ ) break;
 	}
-
-	
+	bool blank = (drawing_background_colour_ != "none");
+	push_front(new FrameBackgroundObject(blank, Colour(drawing_background_colour_)));
 	
 	//Here we have the steps! 	
-	prepareLayout(tree);	
+	prepareLayout(tree);
+	
+	push_back(new FrameForegroundObject(frameIt_, frameColour_, frameStyle_, frameThickness_));
+		
 	
 	if ( items_.empty() )
 	{
@@ -315,13 +323,14 @@ void ViewNode::visit(SceneLayer& tree)
 		}
 	}
 
+	
 	for ( vector<BasicSceneObject*>::iterator item = items_.begin(); item != items_.end(); ++item)  {
+
 		(*item)->visit(tree, components_);
 	}
-	if ( frameHelper_->layoutPtr() )
-	{
-		frameHelper_->layout().frameIt();
-	}
+	
+	//frameHelper_->frameIt();
+	
 
 	if( mode() == interactif )
 	{
@@ -447,8 +456,12 @@ void XmlViewNode::getReady()
 	layout_->y(bottom.percent());
 	layout_->width(width.percent());
 	layout_->height(height.percent());
-	
-	layout_->display(display_);	
+	frameIt_ = border_;
+	frameColour_ = *border_colour_;
+	frameStyle_ = border_style_;
+	frameThickness_ = border_thickness_;
+	drawing_background_colour_ = background_->name();
+	layout_->display(display_);	 
 	layout_->frame(true, border_, *border_colour_, border_style_, border_thickness_, *background_);
 
 	BasicSceneObject::getReady();
@@ -563,6 +576,8 @@ void FortranViewNode::getReady()
 {
 	ASSERT (parent_);
 
+	drawing_background_colour_ = background_->name();
+
 	if ( predefined_ ) {
 			MagDefLibrary library("projections");
 
@@ -621,6 +636,7 @@ void FortranViewNode::getReady()
 	drawing_left_ = x;
 
 
+
 	vaxis_ = vertical_axis_with_;
 	haxis_ = horizontal_axis_height_;
 
@@ -630,7 +646,11 @@ void FortranViewNode::getReady()
 	ParameterManager::set("subpage_y_length_internal", absheight);
 	ParameterManager::set("subpage_x_position_internal", absx);
 	ParameterManager::set("subpage_y_position_internal", absy);
-
+	frameIt_ = frame_;
+	frameColour_ = *frame_colour_;
+	frameStyle_ = frame_line_style_;
+	frameThickness_ = frame_thickness_;
+	
 	layout_->frame(true, frame_, *frame_colour_, frame_line_style_, frame_thickness_, *background_);
 	layout_->clippIt(clipping_);
 
